/**
 * @file    smartptr.h
 
 * @brief   스마트 포인터 구현의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/28
 */
#ifndef _SMARTPTR_H_
#define _SMARTPTR_H_

#include <cstdlib>
#include <functional>
#include <memory>
#include <cassert>

namespace C2SModule_Cocos2dx {
    /**
     * @class _Static_tag
     
     * @brief 스테이틱 타입 테그
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    struct _Static_tag {
    };
    /**
     * @class _Const_tag
     
     * @brief 상수 타입 테그
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    struct _Const_tag {
    };
    /**
     * @class _Dynamic_tag
     
     * @brief 다이나믹 타입 테그
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    struct _Dynamic_tag {
    };
    /**
     * @class add_reference
     
     * @brief 레퍼런스 타입이 아닐 경우 레퍼런스 타입으로 만든다.
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Tp>
    struct add_reference {
        typedef _Tp& type;
    };
    /**
     * @brief 레퍼런스 타입일 경우 그대로 둔다.
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Tp>
    struct add_reference<_Tp&> {
        typedef _Tp& type;
    };
    
    /**
     * @class TSmartPtrBase
     
     * @brief 스마트 포인터의 베이스 타입
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty>
    class TSmartPtrBase {
    public:
        /** 현재 타입의 별칭 */
        typedef TSmartPtrBase<_Ty> _Myt;
        /** 템플릿 인자의 별칭 */
        typedef _Ty _Elem;
        /** 템플릿 인자의 별칭의 별칭 */
        typedef _Elem element_type;
        
        /**
         * @brief default constructor
         
         * @author Byeong Yeol Jeon
         
         */
        TSmartPtrBase() :
        _Ptr(NULL) {
        }
        
        /**
         * @brief 레퍼런스 카운트를 구한다.
         * @return 현재 레퍼런스 카운트
         
         * @author Byeong Yeol Jeon
         
         */
        int32_t use_count() const {
            return (_Ptr ? _Ptr->GetRefCount() : 0);
        }
        
        /**
         * @brief 두 스마트 포인터를 바꾼다.
         
         * @param[in] _Right 대상 스마트 포인터
         
         * @author Byeong Yeol Jeon
         
         */
        void _Swap(TSmartPtrBase& _Right) {
            std::swap(_Ptr, _Right._Ptr);
        }
        
        /**
         * @brief sort용 less 구현
         
         * @param[in] _Right 비교할 대상
         
         * @return true일 경우 작음.
         
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        bool owner_before(const TSmartPtrBase<_Ty2>& _Right) const {
            return (_Ptr < _Right._Ptr);
        }
        /**
         * @brief 현재 실 포인터를 구한다.
         * @return 실 포인터
         
         * @author Byeong Yeol Jeon
         
         */
        _Ty *_Get() const {
            return (_Ptr);
        }
        /**
         * @brief 레퍼런스 카운트를 감소시킨다.
         
         * @author Byeong Yeol Jeon
         
         */
        void _Decref() {
            if (NULL != _Ptr)
                _Ptr->Release();
        }
        /**
         * @brief 현재 소유한 포인터를 해제한다.
         * @author Byeong Yeol Jeon
         
         */
        void _Reset() {
            _Reset(NULL);
        }
        /**
         * @brief 현재 소유한 인스턴스를 다른 인스턴스로 교체한다.
         
         * @param[in] _Other 교체할 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        void _Reset(const TSmartPtrBase<_Ty2>& _Other) {
            _Reset(_Other._Ptr);
        }
        /**
         * @brief 현재 소유한 인스턴스를 다른 인스턴스로 교체한다. static_cast가 가능할 경우 변환하여 포인터를 전달한다.
         
         * @param[in] _Other 교체할 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        void _Reset(const TSmartPtrBase<_Ty2>& _Other,
                    const _Static_tag&) {
            _Reset(static_cast<_Elem *>(_Other._Ptr));
        }
        /**
         * @brief 현재 소유한 인스턴스를 다른 인스턴스로 교체한다. const_cast 해야할 경우 변환하여 전달한다.
         * @param[in] _Other 교체할 인스턴스
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        void _Reset(const TSmartPtrBase<_Ty2>& _Other,
                    const _Const_tag&) {
            _Reset(const_cast<_Elem *>(_Other._Ptr));
        }
        /**
         * @brief 현재 소유한 인스턴스를 다른 인스턴스로 교체한다. dynamic_cast가 가능할 경우 변환하여 포인터를 전달한다.
         
         * @param[in] _Other 교체할 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        void _Reset(const TSmartPtrBase<_Ty2>& _Other,
                    const _Dynamic_tag&) {
            _Elem *_ePtr = dynamic_cast<_Elem *>(_Other._Ptr);
            if (_ePtr)
                _Reset(_ePtr);
            else
                _Reset();
        }
        /**
         * @brief auto_ptr을 스마트 포인터의 인스턴스로 리셋하지 못하도록 만든다.
         
         * @param[in] _Other auto_ptr 인스턴스
         * @remark auto_ptr의 소유권 정책과 스마트 포인터의 소유권 정책은 호환되지 않는다.
         
         * @see <#See#>
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        void _Reset(std::auto_ptr<_Ty2>& _Other) {
            // 모듈이 std를 사용하지 않으므로 여기에선 주석처리
            //throw std::logic_error("invalid call");
            assert(!"invalid call");
        }
        /**
         * @brief 허용 가능한 인스턴스로 리셋한다.
         
         * @param[in] _Other_ptr 입력 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        void _Reset(_Ty *_Other_ptr) {
            if (NULL != _Other_ptr)
                _Other_ptr->AddRef();
            _Reset0(_Other_ptr);
        }
        /**
         * @brief 허용 가능한 인스턴스로 리셋한다. 인자를 0으로 생각
         
         * @param[in] _Other_ptr 입력 인스턴스
         * @author Byeong Yeol Jeon
         
         */
        void _Reset0(_Ty *_Other_ptr) {
            if (NULL != _Ptr)
                _Ptr->Release();
            _Ptr = _Other_ptr;
        }
        
    private:
        /** 소유한 인스턴스 */
        _Ty *_Ptr;
        /** 베이스 타입 */
        template<class _Ty0> friend class TSmartPtrBase;
    };
    /**
     * @class TSmartPtr
     
     * @brief 스마트 포인터
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty>
    class TSmartPtr: public TSmartPtrBase<_Ty> {
    public:
        /** 현재 템플릿 인자의 별칭 */
        typedef _Ty Type;
        /** 현재 타입의 별칭 */
        typedef TSmartPtr<_Ty> _Myt;
        /** 베이스 타입의 별칭 */
        typedef TSmartPtrBase<_Ty> _Mybase;
        /**
         * @brief default constructor
         
         * @author Byeong Yeol Jeon
         
         */
        TSmartPtr() {
        }
        /**
         * @brief constructor
         
         * @param[in] _Px 소유할 대상 포인터
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ux>
        explicit TSmartPtr(_Ux *_Px) {
            _Resetp(_Px);
        }
        /**
         * @brief constructor
         
         * @param[in] _Other 소유할 대상 스마트 포인터의 레퍼런스
         
         * @author Byeong Yeol Jeon
         
         */
        TSmartPtr(const _Myt& _Other) {
            this->_Reset(_Other);
        }
        /**
         * @brief constructor. 템플릿 인자가 다른 스마트 포인터의 소유권을 이전할 경우
         
         * @param[in] _Ohter 템플릿 인자가 다른 스마트 포인터
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        TSmartPtr(const TSmartPtr<_Ty2>& _Other) {
            this->_Reset(_Other);
        }
        /**
         * @brief constructor. auto_ptr로 인스턴스를 생성할 경우 어떤 처리도 하지 않는다.
         
         * @param[in] _Other 입력 auto_ptr
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        TSmartPtr(std::auto_ptr<_Ty2>& _Other) {
        }
        /**
         * @brief constructor. static_cast가 가능할 경우 변환하여 포인터를 전달한다.
         
         * @param[in] _Other 교체할 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        TSmartPtr(const TSmartPtr<_Ty2>& _Other,
                  const _Static_tag& _Tag) {
            this->_Reset(_Other, _Tag);
        }
        /**
         * @brief constructor. const_cast일 경우의 구현
         
         * @param[in] _Other 교체할 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        TSmartPtr(const TSmartPtr<_Ty2>& _Other, const _Const_tag& _Tag) {
            this->_Reset(_Other, _Tag);
        }
        /**
         * @brief constructor. dynamic_cast일 경우의 구현
         
         * @param[in] _Other 교체할 인스턴스
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ty2>
        TSmartPtr(const TSmartPtr<_Ty2>& _Other,
                  const _Dynamic_tag& _Tag) {
            this->_Reset(_Other, _Tag);
        }
        /**
         * @brief destructor
         
         * @author Byeong Yeol Jeon
         
         */
        ~TSmartPtr() {
            this->_Decref();
        }
        /**
         * @brief assign operator
         * @param[in] _Right 할당할 인스턴스
         * @return 현재 인스턴스
         * @author Byeong Yeol Jeon
         */
        _Myt& operator=(const _Myt& _Right) {
            this->_Reset(_Right);
            return (*this);
        }
        /**
         * @brief assign operator
         * @param[in] _Right 할당할 인스턴스
         * @return 현재 인스턴스
         * @author Byeong Yeol Jeon
         */
        template<class _Ty2>
        _Myt& operator=(const TSmartPtr<_Ty2>& _Right) {
            _Reset(_Right);
            return (*this);
        }
        /**
         * @brief assign operator. stl auto ptr은 사용하지 말것.
         * @param[in] _Right 할당할 인스턴스
         * @return 현재 인스턴스
         * @author Byeong Yeol Jeon
         */
        template<class _Ty2>
        _Myt& operator=(std::auto_ptr<_Ty2>& _Right) {
            assert(!"invalid call");
            return (*this);
        }
        /**
         * @brief bool operator
         * @return true일 경우 값이 있음.
         * @author Byeong Yeol Jeon
         
         */
        operator bool() const {
            return NULL != get();
        }
        /**
         * @brief 소유한 포인터를 제거
         
         * @author Byeong Yeol Jeon
         
         */
        void reset() {
            TSmartPtr().swap(*this);
        }
        /**
         * @brief 소유한 포인터를 대상으로 치환
         
         * @param[in] _Px 교체할 포인터
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ux>
        void reset(_Ux *_Px) {
            TSmartPtr(_Px).swap(*this);
        }
        /**
         * @brief 스마트 포인터끼리 소유한 포인터를 교환
         
         * @param[in] _Other 대상 스마트 포인터
         * @author Byeong Yeol Jeon
         
         */
        void swap(_Myt& _Other) {
            this->_Swap(_Other);
        }
        /**
         * @brief 현재 소유한 포인터를 구한다.
         
         * @return 소유한 포인터
         
         * @author Byeong Yeol Jeon
         
         */
        _Ty *get() const {
            return (this->_Get());
        }
        /**
         * @brief 현재 포인터의 값을 참조형으로 얻는다.
         * @return 현재 포인터 값의 참조형
         
         * @author Byeong Yeol Jeon
         
         */
        typename add_reference<_Ty>::type operator*() const {
            return (*this->_Get());
        }
        /**
         * @brief 포인터 접근을 wrapping
         * @return 소유한 포인터
         * @author Byeong Yeol Jeon
         
         */
        _Ty *operator->() const {
            return (this->_Get());
        }
        /**
         * @brief 존재하는 인스턴스가 1개인지 체크
         
         * @return 1개이면 true
         * @author Byeong Yeol Jeon
         
         */
        bool unique() const {
            return (this->use_count() == 1);
        }
        
    private:
        /**
         * @brief 대상 포인터의 레퍼런스를 증가시키고 교체한다.
         
         * @param[in] _Px 교체할 포인터.
         
         * @return <#Return#>
         
         * @remark <#Remark#>
         
         * @see <#See#>
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ux>
        void _Resetp(_Ux *_Px) {
            if (NULL != _Px)
                _Px->AddRef();
            _Resetp0(_Px);
        }
        
    public:
        /**
         * @brief 포인터를 대상 포인터로 교체한다.
         
         * @param[in] _Px 교체할 포인터
         
         * @author Byeong Yeol Jeon
         
         */
        template<class _Ux>
        void _Resetp0(_Ux *_Px) {
            this->_Reset0(_Px);
        }
    };
    /**
     * @param[in] _S1 비교할 대상
     * @param[in] _S2 비교할 대상
     * @return 포인터의 주소가 같으면 true
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    template<class _Ty1, class _Ty2>
    bool operator==(const TSmartPtr<_Ty1>& _S1, const TSmartPtr<_Ty2>& _S2) {
        return (_S1.get() == _S2.get());
    }
    /**
     * @param[in] _S1 비교할 대상
     * @param[in] _S2 비교할 대상
     * @return 포인터의 주소가 다르면 true
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty1, class _Ty2>
    bool operator!=(const TSmartPtr<_Ty1>& _S1, const TSmartPtr<_Ty2>& _S2) {
        return (!(_S1 == _S2));
    }
    /**
     * @param[in] _S1 비교할 대상
     * @param[in] _S2 비교할 대상
     * @return 포인터의 주소가 작으면 true
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    template<class _Ty1, class _Ty2>
    bool operator<(const TSmartPtr<_Ty1>& _S1, const TSmartPtr<_Ty2>& _S2) {
        return (_S1.get() < _S2.get());
    }
    /**
     * @brief greater equal
     
     * @param[in] _S1 비교할 대상
     * @param[in] _S2 비교할 대상
     * @return 포인터의 주소가 크거나 같으면 true
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty1, class _Ty2>
    bool operator>=(const TSmartPtr<_Ty1>& _S1, const TSmartPtr<_Ty2>& _S2) {
        return (!(_S1 < _S2));
    }
    /**
     * @brief greater
     
     * @param[in] _S1 비교할 대상
     * @param[in] _S2 비교할 대상
     * @return 포인터의 주소가 크면 true
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty1, class _Ty2>
    bool operator>(const TSmartPtr<_Ty1>& _S1, const TSmartPtr<_Ty2>& _S2) {
        return (_S2 < _S1);
    }
    /**
     * @brief less equal
     * @param[in] _S1 비교할 대상
     * @param[in] _S2 비교할 대상
     * @return 포인터의 주소가 작거나 같으면 true
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty1, class _Ty2>
    bool operator<=(const TSmartPtr<_Ty1>& _S1, const TSmartPtr<_Ty2>& _S2) {
        return (!(_S2 < _S1));
    }
    /**
     * @brief 두 스마트포인터의 포인터를 교환
     
     * @param[in] _Left 대상 스마트포인터
     * @param[in] _Right 대상 스마트포인터
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class _Ty>
    void swap(TSmartPtr<_Ty>& _Left, TSmartPtr<_Ty>& _Right) {
        _Left.swap(_Right);
    }
    /**
     * @brief 스마트포인터의 포인터를 static_cast한다.
     
     * @param[in] _Other 대상 스마트 포인터
     
     * @return 캐스팅된 스마트 포인터
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    template<class _Ty1, class _Ty2>
    TSmartPtr<_Ty1> static_pointer_cast(const TSmartPtr<_Ty2>& _Other) {
        return (TSmartPtr<_Ty1>(_Other, _Static_tag()));
    }
    /**
     * @brief 스마트포인터의 포인터를 const_cast한다.
     
     * @param[in] _Other 대상 스마트 포인터
     
     * @return 캐스팅된 스마트 포인터
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    template<class _Ty1, class _Ty2>
    TSmartPtr<_Ty1> const_pointer_cast(const TSmartPtr<_Ty2>& _Other) {
        return (TSmartPtr<_Ty1>(_Other, _Const_tag()));
    }
    /**
     * @brief 스마트포인터의 포인터를 dynamic_cast한다.
     
     * @param[in] _Other 대상 스마트 포인터
     
     * @return 캐스팅된 스마트 포인터
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    template<class _Ty1, class _Ty2>
    TSmartPtr<_Ty1> dynamic_pointer_cast(const TSmartPtr<_Ty2>& _Other) {
        return (TSmartPtr<_Ty1>(_Other, _Dynamic_tag()));
    }
}
#define DefSmartPtr( CLASS ) typedef C2SModule_Cocos2dx::TSmartPtr< CLASS > CLASS##Ptr

#endif /* _SMARTPTR_H_ */
