/**
 * @brief 플랫폼 사용 편의를 위한 헬퍼 기능들의 모음
 *
 * @author joosangkang
 *
 * @defgroup PlatformHelper
 * @{
 *
 */

#ifndef HIVE_PlatformHelper_hpp
#define HIVE_PlatformHelper_hpp

#include <stdio.h>

NS_HIVE_BEGIN

enum class PlatformShareType;
enum class PlatformShareDataType;
class PlatformShare;

/**
 * 플랫폼에서 지원하는 기능을 제공하는 클래스이다.
 * @ingroup PlatformHelper
 */
class HIVESDK_DLLEXPORT PlatformHelper
{
public:
    
    /**
     * @brief Share 관련 동작이 완료되었을 때 호출됨. (shareText, shareMedia)
     *
     * @param isSuccess Share 결과.
     *
     * @ingroup PlatformHelper
     */
    typedef std::function<void(bool isSuccess)> onHIVEPlatformHelperShareHandler;
    
    /**
     * @brief 하나 또는 복수 개의 미디어(이미지, 오디오, 비디오) 또는 텍스트를 공유한다.
     *
     * @param platformShare 공유할 미디어의 Uri(s)와 공유할 텍스트
     */
    static void share(PlatformShare platformShare, onHIVEPlatformHelperShareHandler handler);

    /**
     * @brief Android에서 재요청된 OS 권한동의에 대한 결과 값을 반환한다.
     *
     * @param granted 사용자가 수락한 권한 리스트, denied 사용자가 거부한 권한 리스트
     *
     * @ingroup PlatformHelper
     */
    typedef std::function<void(ResultAPI const & result, std::vector<std::string> const & granted, std::vector<std::string> const & denied)> onRequestUserPermissionsHandler;

    /**
     * @brief Android에서 사용자에게 OS 권한을 재요청.
     *
     * @param requests Android에서 요청할 권한들을 포함한 리스트.
     *
     * @ingroup PlatformHelper
     */
    static void requestUserPermissions(std::vector<std::string> requests, onRequestUserPermissionsHandler handler);
    
    // Native 영역에서 호출된 요청을 처리하기 위한 플러그인 내부 코드
    static void executeEngine(picojson::value jsonParam);
};

/**
 * \~korean  공유 타입
 *
 * \~english Types of Platform share
 * \~
 * @ingroup PlatformHelper
 */
enum class PlatformShareType
{
    TEXT     = 1       ///< \~korean 텍스트  \~english  Text
    , MEDIA  = 2      ///< \~korean 미디어       \~english Media
};

/**
 * \~korean  공유 데이터 타입
 *
 * \~english Types of Platform share data
 * \~
 * @ingroup PlatformHelper
 */
enum class PlatformShareDataType
{
    SUBJECT = 0,   ///< \~korean 주제    \~english  Subject
    TEXT,         ///< \~korean 텍스트       \~english Text
    MEDIA,         ///< \~korean 미디어       \~english Media
};

class HIVESDK_DLLEXPORT PlatformShare
{
public:
    PlatformShare();
    ~PlatformShare();
    
    PlatformShareType getShareType();
    int getShareTypeInt();
    void setShareType(PlatformShareType shareType);
    
    std::string getSubject();
    void setSubject(std::string subject);
    
    std::string getText();
    void setText(std::string text);
    
    std::vector<std::string> getMedia();
    void setMedia(std::vector<std::string> paths);
    
private:
    PlatformShareType shareType;
    
    std::string subject;
    std::string text;
    std::vector<std::string> paths;
};

NS_HIVE_END



#endif /* HIVE_PlatformHelper_hpp */


/** @} */
