/**
 * @file    Module_Cocos2d-x.h
 
 * @brief   c++ 지원 모듈의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/25
 */
#ifndef _MODULE_COCOS2D_X_H_
#define _MODULE_COCOS2D_X_H_

#include "../C2SModuleErrorCode.h"
#include "ModuleHelper_Cocos2d-x.h"
#include "smartptr.h"
#include "refobject.h"

#if __OBJC__
#elif __ANDROID__
#include <jni.h>
#endif

#include <map>
#include <vector>
#include <list>
#include <deque>
#include <string>
#include "adapt_tuple.h"

/**
 * @namespace C2SModule_Cocos2dx
 
 * @brief c++ 지원 클래스들의 네임스페이스
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 
 */
namespace C2SModule_Cocos2dx {
    /**
     * @class C2SModuleErrorDesc
     
     * @brief 에러 정보를 담는 클래스
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    class C2SModuleErrorDesc : public C2SModule_Cocos2dx::refobject {
    public:
        /**
         * @brief 에러 코드를 구한다.
         * @return 에러 코드
         * @author Byeong Yeol Jeon
         
         */
        virtual C2SModuleErrorCode GetCode() const = 0;
        /**
         * @brief 에러 문자열을 구한다.
         * @return 에러 문자열
         * @author Byeong Yeol Jeon
         
         */
        virtual const char* GetMessage() const = 0;
    };
    
    /**
     * @brief C2SModuleError의 스마트 포인터
     * @ingroup Cocos2d-x
     */
    typedef C2SModule_Cocos2dx::TSmartPtr<C2SModuleErrorDesc> C2SModuleError;
    
    //====================================================================================================================
    /**
     * @class C2SModuleCompletionHandler
     
     * @brief 유저가 정의할 수 있는 결과 통지 델리게이트의 형식
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    class C2SModuleCompletionHandler : public C2SModule_Cocos2dx::refobject {
    public:
        /**
         * @brief 결과 통지시 실행되는 함수
         
         * @param[in] pstrJson 결과 인자
         * @param[in] error 발생한 에러
         
         * @author Byeong Yeol Jeon
         
         */
        virtual void onComplete(const char* pstrJson, const C2SModuleErrorDesc* error) const = 0;
    };
    
    // foward decl
    class C2SModuleParameterMap;
    class C2SModuleParameterArray;
    
    /**
     * @class C2SModuleParameterMap
     
     * @brief 키 밸류 페어를 지원하는 커스텀 map
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    class C2SModuleParameterMap {
    public:
#if ADAPT_TUPLE
        /**
         * @brief 튜플을 인자로 받는 constructor
         
         * @param[in] t 인자들의 리스트
         * @author Byeong Yeol Jeon
         
         */
        template<class... T>
        inline C2SModuleParameterMap(T&&... t) : C2SModuleParameterMap()
        {
            ParseTuple(std::make_tuple(t...));
        }
#endif
        /**
         * @brief default constructor
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleParameterMap();
        /**
         * @brief default destructor
         * @author Byeong Yeol Jeon
         
         */
        ~C2SModuleParameterMap();
        /**
         * @brief 키에 bool 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], bool arg);
        /**
         * @brief 키에 8bit signed 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], int8_t arg);
        /**
         * @brief 키에 8bit unsigned 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], uint8_t arg);
        /**
         * @brief 키에 16bit signed 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], int16_t arg);
        /**
         * @brief 키에 16bit unsigned 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], uint16_t arg);
        /**
         * @brief 키에 32bit signed 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], int32_t arg);
        /**
         * @brief 키에 32bit unsigned 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], uint32_t arg);
        /**
         * @brief 키에 64bit signed 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], int64_t arg);
        /**
         * @brief 키에 16bit unsigned 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], uint64_t arg);
        /**
         * @brief 키에 문자열 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], const char arg[]);
        /**
         * @brief 키에 raw 이미지 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], C2SModuleImage& arg);
        /**
         * @brief 키에 map 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], const C2SModuleParameterMap& arg);
        /**
         * @brief 키에 array 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], const C2SModuleParameterArray& arg);
        
#if __OBJC__
        /**
         * @brief 키에 objc 타입 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], id arg);
#elif __ANDROID__
        /**
         * @brief 키에 android 오브젝트 타입 값을 넣는다
         * @param[in] key 키
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Set(const char key[], jobject arg);
#endif
        /**
         * @brief 현재 소유한 raw 데이터를 구한다.
         * @return 현재 소유한 raw 데이터
         * @author Byeong Yeol Jeon
         */
        void* Get() const;
        
    private:
        /** copy prevent */
        C2SModuleParameterMap(const C2SModuleParameterMap&);
        /** copy prevent */
        C2SModuleParameterMap& operator=(const C2SModuleParameterMap&);
        
        /** 현재 소유한 raw 데이터의 주소 */
        void* container;
        
#if ADAPT_TUPLE
        /**
         * @brief  튜플을 분석하여 맵에 키 밸류로 넣는다.
         
         * @param[in] arg 입력 튜플
         
         * @author Byeong Yeol Jeon
         
         */
        template<typename... T>
        inline void ParseTuple(const std::tuple<T...>& arg)
        {
            typedef std::tuple< T... > arg_type;
            
            TupleMapIterator< C2SModuleParameterMap, C2SModuleParameterArray, std::tuple_size< arg_type >::value >::Pred(*this, arg);
        }
#endif
    };
    
    /**
     * @class C2SModuleParameterArray
     
     * @brief 커스텀 map에서 사용하는 array 타입
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    class C2SModuleParameterArray {
    public:
#if ADAPT_TUPLE
        /**
         * @brief 튜플을 인자로 받는 constructor
         
         * @param[in] t 인자들의 리스트
         * @author Byeong Yeol Jeon
         
         */
        template<class... T>
        inline C2SModuleParameterArray(T&&... t) : C2SModuleParameterArray()
        {
            ParseTuple(std::make_tuple(t...));
        }
#endif
        /**
         * @brief default constructor
         
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleParameterArray();
        /**
         * @brief default destructor
         * @author Byeong Yeol Jeon
         
         */
        ~C2SModuleParameterArray();
        
        /**
         * @brief bool값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(bool arg);
        /**
         * @brief 8bit signed 값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(int8_t arg);
        /**
         * @brief 8bit unsigned 값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(uint8_t arg);
        /**
         * @brief 16bit signed 값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(int16_t arg);
        /**
         * @brief 16bit unsigned 값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(uint16_t arg);
        /**
         * @brief 32bit signed  값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(int32_t arg);
        /**
         * @brief 32bit unsigned  값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(uint32_t arg);
        /**
         * @brief 64bit signed 값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(int64_t arg);
        /**
         * @brief 64bit unsigned 값을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(uint64_t arg);
        /**
         * @brief 문자열을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(const char arg[]);
        /**
         * @brief 커스텀 map을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(const C2SModuleParameterMap& arg);
        
#if __OBJC__
        /**
         * @brief objc타입을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(id arg);
#elif __ANDROID__
        /**
         * @brief android object 타입을 array에 추가한다.
         * @param[in] arg 값
         * @author Byeong Yeol Jeon
         */
        void Add(jobject arg);
#endif
        /**
         * @brief 현재 소유한 raw 데이터를 구한다.
         * @return 현재 소유한 raw 데이터
         * @author Byeong Yeol Jeon
         */
        void* Get() const;
        
    private:
        /** copy prevent */
        C2SModuleParameterArray(const C2SModuleParameterArray&);
        /** copy prevent */
        C2SModuleParameterArray& operator=(const C2SModuleParameterArray&);
        
        /** 현재 소유한 raw 데이터의 주소 */
        void* container;
        
#if ADAPT_TUPLE
        /**
         * @brief  튜플을 분석하여 array에 추가한다.
         
         * @param[in] arg 입력 튜플
         
         * @author Byeong Yeol Jeon
         
         */
        template<typename... T>
        inline void ParseTuple(const std::tuple<T...>& arg)
        {
            typedef std::tuple< T... > arg_type;
            
            TupleArrayIterator< C2SModuleParameterMap, C2SModuleParameterArray, std::tuple_size< arg_type >::value >::Pred(*this, arg);
        }
#endif
    };
    
    typedef C2SModuleParameterMap C2SModuleParameter;
}
/**
 * @brief Social Initialize의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSocial_Initialize(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Social Initialize의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSocial_Initialize(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Social Show의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSocial_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Social Show의 c++ 인터페이스 
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSocial_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Social Info의 c++ 인터페이스 Added v1.10.0
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Daun Joung
 */
bool C2SModuleSocial_Info(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social Info의 c++ 인터페이스 Added v1.10.0
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Daun Joung
 */
bool C2SModuleSocial_Info(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Social SetGameInfo의 c++ 인터페이스 Added v1.10.0
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Daun Joung
 */
bool C2SModuleSocial_SetGameInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social SetGameInfo의 c++ 인터페이스 Added v1.10.0
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Daun Joung
 */
bool C2SModuleSocial_SetGameInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social UpdateGameLanguage의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Seokjin Yong
 */
bool C2SModuleSocial_UpdateGameLanguage(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social Initialize의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Seokjin Yong
 */
bool C2SModuleSocial_UpdateGameLanguage(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social UpdateServerId의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Seokjin Yong
 */
bool C2SModuleSocial_UpdateServerId(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social UpdateServerId의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Seokjin Yong
 */
bool C2SModuleSocial_UpdateServerId(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                    C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Social SendData의 c++ 인터페이스 Added v1.10.0
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Daun Joung
 */
bool C2SModuleSocial_SendData(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Social Info의 c++ 인터페이스 Added v1.10.0
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Daun Joung
 */
bool C2SModuleSocial_SendData(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                 C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Session의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Session(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Session의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Session(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Login의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Login(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Login의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Login(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Logout의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Logout(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Logout의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Logout(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                         C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Unregister의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Unregister(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Unregister의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Unregister(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns User의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_User(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns User의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_User(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Friends의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Friends(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Friends의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Friends(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Message의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Message(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Message의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Message(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Post의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Post(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Post의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Post(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Show의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns Show의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns SetGuestUserInfo의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_SetGuestUserInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns SetGuestUserInfo의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModuleSns_SetGuestUserInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns UserDelete의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author choiBK
 */
bool C2SModuleSns_UserDelete(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Sns UserDelete의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author choiBK
 */
bool C2SModuleSns_UserDelete(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                         C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push Register의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePush_Register(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push Register의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePush_Register(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                            C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push Unregister의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePush_Unregister(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push Unregister의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePush_Unregister(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                              C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push Description의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePush_Description(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push Description의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePush_Description(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                               C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push RequestPermission의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author sjyong
 */
bool C2SModulePush_RequestPermission(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Push RequestPermission의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author sjyong
 */
bool C2SModulePush_RequestPermission(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Promotion Show의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePromotion_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Promotion Show의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Byeong Yeol Jeon
 */
bool C2SModulePromotion_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Promotion Info의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModulePromotion_Info(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Promotion Info의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModulePromotion_Info(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

//1.9.0
/**
 * @brief Promotion Acquisition의 c++ 인터페이스 (added v1.9.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Choi BK
 */
bool C2SModulePromotion_Acquisition(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Promotion Acquisition의 c++ 인터페이스 (added v1.9.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Choi BK
 */
bool C2SModulePromotion_Acquisition(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
 * @remark
 * - "campaignType" : NSString*.
 *      - "event"  : 이벤트 타입
 *      - "notice" : 공지 타입
 * - "bannerType" : NSString*.
 *      - "great"   : 대배너
 *      - "small"   : 일반배너 (띠배너)
 *      - "rolling" : 롤링 배너
 * @author daunjoung
 */
bool C2SModulePromotion_GetBannerInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
 * @remark
 * - "campaignType" : NSString*.
 *      - "event"  : 이벤트 타입
 *      - "notice" : 공지 타입
 * - "bannerType" : NSString*.
 *      - "great"   : 대배너
 *      - "small"   : 일반배너 (띠배너)
 *      - "rolling" : 롤링 배너
 * @author daunjoung
 */
bool C2SModulePromotion_GetBannerInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

// Engagement
/**
 * @brief UserEngagement ProcessScheme의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Hysterit
 */
bool C2SModuleEngagement_ProcessScheme(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief UserEngagement ProcessScheme의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Hysterit
 */
bool C2SModuleEngagement_ProcessScheme(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief UserEngagement GetEvents의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Hysterit
 * Deprecated in HIVE SDK 1.11.0
 */
//bool C2SModuleEngagement_GetEvents(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief UserEngagement GetEvents의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Hysterit
 * Deprecated in HIVE SDK 1.11.0
 */
//bool C2SModuleEngagement_GetEvents(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
//                                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief UserEngagement ConsumeCoupon의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Hysterit
 */
bool C2SModuleEngagement_ConsumeCoupon(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief UserEngagement ConsumeCoupon의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author Hysterit
 */
bool C2SModuleEngagement_ConsumeCoupon(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);


/**
 * @brief UserEngagement SetReady의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author DaunJung
 */
bool C2SModuleEngagement_SetReady(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief UserEngagement SetReady의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author DaunJung
 */
bool C2SModuleEngagement_SetReady(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Tracker SendEvent의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */


bool C2SModuleTracker_SendEvent(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Tracker SendEvent의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author cocororo
 */
bool C2SModuleTracker_SendEvent(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Tracker SetEnable의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleTracker_SetEnable(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Tracker SetEnable의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author cocororo
 */
bool C2SModuleTracker_SetEnable(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Hive Promotion(Promotion2.0) Show의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Hive Promotion(Promotion2.0) Show의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);


/**
 * @brief Hive Promotion(Promotion2.0) Info의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_Info(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Hive Promotion(Promotion2.0) Info의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_Info(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);


/**
 * @brief Hive Promotion(Promotion2.0) GetOfferwallState의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_GetOfferwallState(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Hive Promotion(Promotion2.0) GetOfferwallState의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_GetOfferwallState(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);


/**
 * @brief Hive Promotion(Promotion2.0) SetAdditionalInfo의 c++ 인터페이스 (added v1.5.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_SetAdditionalInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Hive Promotion(Promotion2.0) SetAdditionalInfo의 c++ 인터페이스 (added v1.5.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleHivePromotion_SetAdditionalInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief Hive Promotion(Promotion2.0) UpdatePromotionData의 c++ 인터페이스 (added v1.19.0)
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author jschoi
 */
bool C2SModuleHivePromotion_UpdatePromotionData(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief Hive Promotion(Promotion2.0) UpdatePromotionData의 c++ 인터페이스 (added v1.19.0)
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author jschoi
 */
bool C2SModuleHivePromotion_UpdatePromotionData(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                              C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
 * @remark
 * - "campaignType" : NSString*.
 *      - "event"  : 이벤트 타입
 *      - "notice" : 공지 타입
 * - "bannerType" : NSString*.
 *      - "great"   : 대배너
 *      - "small"   : 일반배너 (띠배너)
 *      - "rolling" : 롤링 배너
 * @author daunjoung
 */
bool C2SModuleHivePromotion_GetBannerInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
 * @remark
 * - "campaignType" : NSString*.
 *      - "event"  : 이벤트 타입
 *      - "notice" : 공지 타입
 * - "bannerType" : NSString*.
 *      - "great"   : 대배너
 *      - "small"   : 일반배너 (띠배너)
 *      - "rolling" : 롤링 배너
 * @author daunjoung
 */
bool C2SModuleHivePromotion_GetBannerInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

#if __OBJC__

#elif __ANDROID__

bool C2SModuleSns_PGSLoginProc(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
bool C2SModuleSns_PGSLoginProc(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
								C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

bool C2SModuleSns_Connect(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
bool C2SModuleSns_Connect(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
								C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
#endif

int GetOfferwallState();
const char* GetLoginType();
const char* GetHiveCountry(); // 하이브에서 판단한 국가 코드를 리턴
const char* GetChannel();
void SetChannel(const char* channel);

/**
 * @brief InappV4 ItemInfo 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_ItemInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 ItemInfo 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_ItemInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                               C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 Purchase 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_Purchase(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 Purchase 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_Purchase(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                               C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 Restore 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_Restore(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 Restore 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_Restore(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                              C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief InappV4 TransactionFinish 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_TransactionFinish(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 TransactionFinish 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_TransactionFinish(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 *
 * @brief InappV4 GetSelectedMarket 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_GetSelectedMarket(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 GetSelectedMarket 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_GetSelectedMarket(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 *
 * @brief InappV4 GetAccountUuid 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_GetAccountUuid(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 GetAccountUuid 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_GetAccountUuid(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 *
 * @brief InappV4 MarketConnect 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_MarketConnect(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 MarketConnect 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_MarketConnect(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 *
 * @brief InappV4 ShowInAppMessages 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_ShowInAppMessages(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 ShowInAppMessages 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_ShowInAppMessages(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                     C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

#ifdef __ANDROID__
/**
 * @brief InappV4 SelectMarket 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_SelectMarket(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 SelectMarket 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_SelectMarket(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief InappV4 RequestBalance 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_RequestBalance(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 RequestBalance 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_RequestBalance(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);

/**
 * @brief InappV4 ChargeBalance 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_ChargeBalance(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 ChargeBalance 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_ChargeBalance(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
#endif // __ANDROID__

#if !__ANDROID__
/**
 * @brief InappV4 CheckPromotePurchase 의 c++ 인터페이스
 * @param[in] pstrJson 입력 json 문자열
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_CheckPromotePurchase(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
/**
 * @brief InappV4 CheckPromotePurchase 의 c++ 인터페이스
 * @param[in] arg 입력 커스텀 map
 * @param[in] pkError 실행시 발생한 에러를 받을 인스턴스의 주소
 * @param[in] completionHandler 결과를 통지 받을 델리게이트. NULL 이면 공통 델리게이트로 결과가 전달된다.
 * @return true일 경우 성공. false일 경우 실패
 * @ingroup Cocos2d-x
 * @author pnpsinki
 */
bool C2SModuleInappV4_CheckPromotePurchase(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler = NULL);
#endif //!__ANDROID__

#endif
