/**
 * @file    ModuleHelper_Cocos2d-x.cpp
 
 * @brief   c++ 사용시 제공되는 편의 클래스의 소스
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/25
 */
#include "ModuleHelper_Cocos2d-x.h"
#include <cstdlib>

#if __OBJC__
#import <UIKit/UIImage.h>
#endif

namespace C2SModule_Cocos2dx {
    C2SModuleImageDesc::C2SModuleImageDesc() {
    }
    
    C2SModuleImageDesc::~C2SModuleImageDesc() {
    }

#if __OBJC__
    /**
     * @class C2SModuleImage_iOS
     * @brief C2SModuleImage의 iOS 구현
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x iOS
     */
    class C2SModuleImage_iOS: public C2SModuleImageDesc {
    public:
        /**
         * @brief default constructor
         
         * @param[in] arg 입력 UIImage
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleImage_iOS(UIImage* arg)
        {
            data = arg;
        }
        /**
         * @brief default destructor
         * @author Byeong Yeol Jeon
         
         */
        virtual ~C2SModuleImage_iOS() {
        }
        /**
         * @brief 현재 소유한 UIImage를 구한다.
         * @return 현재 소유한 UIImage
         * @author Byeong Yeol Jeon
         
         */
        virtual void* Get() const {
            return (__bridge void*)data;
        }
        
        /** 현재 소유한 UIImage */
        UIImage* data;
    };
    
#elif __ANDROID__
    
#include <jni.h>
#include <android/log.h>


#ifdef __cplusplus
extern "C" {
#endif
    extern JavaVM* C2SModuleJavaVMInstance;

    extern jclass C2SModuleBitmapClass;
    extern jclass C2SModuleBitmapFactoryClass;
    extern jclass C2SModuleSocialClass;

#ifdef __cplusplus
}
#endif

    /**
     * @class C2SModuleImage_Android
     * @brief C2SModuleImage의 Android 구현
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x Android
     */
    class C2SModuleImage_Android: public C2SModuleImageDesc {
    public:
        /**
         * @brief default constructor
         
         * @param[in] arg 입력 bitmap
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleImage_Android(jobject arg) {
            JNIEnv* env;
            if (JNI_OK
                == C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                                   JNI_VERSION_1_6)) {
                    data = env->NewGlobalRef(arg);
                }
        }
        /**
         * @brief default destructor
         * @author Byeong Yeol Jeon
         */
        virtual ~C2SModuleImage_Android() {
            if (NULL != data) {
                JNIEnv* env;
                if (JNI_OK
                    == C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                                       JNI_VERSION_1_6)) {
                        env->DeleteGlobalRef(data);
                    }
            }
        }
        /**
         * @brief 현재 소유한 bitmap을 구한다.
         * @return 현재 소유한 bitmap
         * @author Byeong Yeol Jeon
         
         */
        virtual void* Get() const {
            return reinterpret_cast<void*>(data);
        }
        
        /// bitmap 인스턴스
        jobject data;
    };
#endif
    
    namespace C2SModuleImageLoader {
        C2SModuleImage FromEncodedData(const void* imageData, int32_t length) {
            if (0 >= length || NULL == imageData)
                return C2SModuleImage();
#if __OBJC__
            return C2SModuleImage(new C2SModuleImage_iOS([UIImage imageWithData:[NSData dataWithBytes:imageData length:length]]));
#elif __ANDROID__
            JNIEnv* env;
            if (JNI_OK
                != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6))
                return C2SModuleImage();
            
            jmethodID methodId = env->GetStaticMethodID(C2SModuleBitmapFactoryClass,
                                                        "decodeByteArray", "([BII)Landroid/graphics/Bitmap;");
            
            jbyteArray arg = env->NewByteArray(length);
            env->SetByteArrayRegion(arg, 0, length,
                                    reinterpret_cast<const jbyte*>(imageData));
            jobject ret = env->CallStaticObjectMethod(C2SModuleBitmapFactoryClass,
                                                      methodId, arg, 0, length);
            
            env->DeleteLocalRef(arg);
            C2SModuleImage_Android* pkRet = new C2SModuleImage_Android(ret);
            env->DeleteLocalRef(ret);
            
            return C2SModuleImage(pkRet);
#else
            return C2SModuleImage();
#endif
        }
        
        C2SModuleImage FromRawData(const void* rawImageData, int32_t width, int32_t height) {
            if (NULL == rawImageData)
                return C2SModuleImage();
#if __OBJC__
            CGDataProviderRef ref = CGDataProviderCreateWithData(NULL, rawImageData, width*height*4, NULL);
            CGImageRef iref = CGImageCreate(width,height,8,32,width*4,CGColorSpaceCreateDeviceRGB(),
                                            kCGBitmapByteOrderDefault,ref,NULL, true, kCGRenderingIntentDefault);
            uint8_t* pixels = new uint8_t[width*height*4];
            
            CGContextRef context = CGBitmapContextCreate(&pixels[0], width, height, 8, width*4, CGImageGetColorSpace(iref),
                                                         kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Big);
            CGContextTranslateCTM(context, 0.0, 0.0);
            CGContextScaleCTM(context, 1.0, 1.0);
            CGContextDrawImage(context, CGRectMake(0.0, 0.0, width, height), iref);
            CGImageRef outputRef = CGBitmapContextCreateImage(context);
            
            UIImage* image = [UIImage imageWithCGImage:outputRef];
            CGImageRelease(iref);
            CGImageRelease(outputRef);
            CGContextRelease(context);
            CGDataProviderRelease(ref);
            
            delete[] pixels;
            return C2SModuleImage(new C2SModuleImage_iOS(image));
#elif __ANDROID__
            JNIEnv* env;
            if (JNI_OK
                != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6))
                return C2SModuleImage();
            
            jmethodID methodId = env->GetStaticMethodID(C2SModuleSocialClass,
                                                        "createBitmap", "([III)Landroid/graphics/Bitmap;");
            
            if (NULL == methodId) {
                __android_log_print(ANDROID_LOG_ERROR, "CreateFromPNG",
                                    "NULL == methodId");
            }
            jintArray arg = env->NewIntArray(width * height);
            env->SetIntArrayRegion(arg, 0, width * height,
                                   reinterpret_cast<const jint*>(rawImageData));
            
            jobject ret = env->CallStaticObjectMethod(C2SModuleBitmapClass, methodId,
                                                      arg, width, height);
            
            env->DeleteLocalRef(arg);
            C2SModuleImage_Android* pkRet = new C2SModuleImage_Android(ret);
            env->DeleteLocalRef(ret);
            
            return C2SModuleImage(pkRet);
#else
            return C2SModuleImage();
#endif
        }
        
        C2SModuleImage FromFile(const char filePath[]) {
            if (NULL == filePath)
                return C2SModuleImage();
#if __OBJC__
            return C2SModuleImage(new C2SModuleImage_iOS([UIImage imageWithData:[NSData dataWithContentsOfFile:[NSString stringWithUTF8String:filePath]]]));
#elif __ANDROID__
            JNIEnv* env;
            if (JNI_OK
                != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6))
                return C2SModuleImage();
            
            jmethodID methodId = env->GetStaticMethodID(C2SModuleBitmapFactoryClass,
                                                        "decodeFile", "(Ljava/lang/String)Landroid/graphics/Bitmap;");
            
            jstring arg = env->NewStringUTF(filePath);
            jobject ret = env->CallStaticObjectMethod(C2SModuleBitmapFactoryClass,
                                                      methodId, arg);
            
            env->DeleteLocalRef(arg);
            C2SModuleImage_Android* pkRet = new C2SModuleImage_Android(ret);
            env->DeleteLocalRef(ret);
            
            return C2SModuleImage(pkRet);
#else
            return C2SModuleImage();
#endif
        }
    }
}
