/**
 * @file    C2SModuleSecureVariable.h
 
 * @brief   변수 암호화 모듈의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x Protection
 * @date    2014/04/28
 */
#ifndef _C2SMODULESECUREVARIABLE_H_
#define _C2SMODULESECUREVARIABLE_H_

#include "../C2SModuleErrorCode.h"
#include <stdint.h>
/**
 * @brief 변수 암호화 오브젝트의 핸들 타입
 * @author Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x Protection
 */
typedef void* C2SModuleSecureId;

#if __cplusplus
extern "C" {
#endif
    /**
     * @brief 변수 암호화 오브젝트를 생성한다.
     * @param[in, out] outSecureId 오브젝트 핸들
     * @return 발생한 에러의 코드
     * @ingroup iOS Android Cocos2d-x Protection
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableCreate(C2SModuleSecureId* outSecureId);
    /**
     * @brief 저장된 데이터를 가져온다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 받을 데이터의 버퍼
     * @param[in] length 받을 데이터 버퍼의 길이
     * @return 발생한 에러 코드
     * @author Byeong Yeol Jeon
     * @ingroup iOS Android Cocos2d-x Protection
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet(C2SModuleSecureId secureId, void* data, uint32_t length);
    /**
     * @brief 데이터를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장할 데이터
     * @param[in] length 저장할 데이터의 길이
     * @return 발생한 에러의 코드
     * @author Byeong Yeol Jeon
     * @ingroup iOS Android Cocos2d-x Protection
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet(C2SModuleSecureId secureId, const void* data, uint32_t length);
    /**
     * @brief 암호화 데이터를 복사한다.
     * @param[in] secureId 데이터가 복사 될 오브젝트 핸들
     * @param[in] srcId 데이터의 원본 오브젝트 핸들
     * @return 발생한 에러의 코드
     * @author Byeong Yeol Jeon
     * @ingroup iOS Android Cocos2d-x Protection
     */
    C2SModuleErrorCode C2SModuleSecureVariableCopy(C2SModuleSecureId secureId, C2SModuleSecureId srcId);
    /**
     * @brief 생성된 암호화 오브젝트를 제거한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @return 발생한 에러 코드
     * @author Byeong Yeol Jeon
     * @ingroup iOS Android Cocos2d-x Protection
     */
    C2SModuleErrorCode C2SModuleSecureVariableRelease(C2SModuleSecureId* secureId);
#if __cplusplus
}
#endif

#if __cplusplus
/**
 * @class DefaultBrokenPolicy
 * @brief 변수의 무결성이 유효하지 않을 떄 실행되는 functor
 * @author Byeong Yeol Jeon
 */
struct DefaultBrokenPolicy
{
    /**
     * @brief function operator
     * @author Byeong Yeol Jeon
     */
	void operator() (void) const
	{
	}
};

/**
 * @class C2SModuleSecureVariable
 
 * @brief 변수 암호화를 c atomic type별로 구현한 템플릿 클래스
 
 * @remark 기본 무결성 체크에 대한 정책으로 DefaultBrokenPolicy을 사용한다.
 * @author Byeong Yeol Jeon
 
 */
template< typename T, class BrokenPolicy = DefaultBrokenPolicy>
class C2SModuleSecureVariable
{
public:
    /**
     * @brief default constructor
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleSecureVariable() {
        C2SModuleSecureVariableCreate(&varId);
	}
    /**
     * @brief assign constructor. 다른 atomic type의 C2SModuleSecureVariable로 현재 클래스를 생성
     
     * @param[in] src 다른 atomic type의 C2SModuleSecureVariable
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable(const C2SModuleSecureVariable<U>& src) : C2SModuleSecureVariable() {
        T tmp;
        if(C2SModuleErrorCode_Broken == C2SModuleSecureVariableGet(src.varId, &tmp, sizeof(tmp)))
        {
        	broken();
        }
        C2SModuleSecureVariableSet(varId, src.varId);
	}
    /**
     * @brief assign constructor
     
     * @param[in] src 동일한 atomic type의 클래스
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable(const C2SModuleSecureVariable& src) : C2SModuleSecureVariable() {
        C2SModuleSecureVariableCopy(varId, src.varId);
	}
    /**
     * @brief 8bit signed constructor
     
     * @param[in] iValue 8bit signed value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(int8_t iValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(iValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 16bit signed constructor
     
     * @param[in] iValue 16bit signed value
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(int16_t iValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(iValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 32bit signed constructor
     
     * @param[in] iValue 32bit signed value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(int32_t iValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(iValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 64bit signed constructor
     
     * @param[in] iValue 64bit signed value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(int64_t iValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(iValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 8bit unsigned constructor
     
     * @param[in] uValue 8bit unsigned value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(uint8_t uValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(uValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 16bit unsigned constructor
     
     * @param[in] uValue 16bit unsigned value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(uint16_t uValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(uValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 32bit unsigned constructor
     
     * @param[in] uValue 32bit unsigned value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(uint32_t uValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(uValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 64bit unsigned constructor
     
     * @param[in] uValue 64bit unsigned value
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(uint64_t uValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(uValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 32bit float constructor
     
     * @param[in] fValue 32bit float value
     
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(float fValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(fValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief 64bit float constructor
     
     * @param[in] fValue 64bit float value
     * @author Byeong Yeol Jeon
     
     */
	explicit C2SModuleSecureVariable(double fValue) : C2SModuleSecureVariable() {
        T tmp(static_cast<T>(fValue));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(T));
	}
    /**
     * @brief destructor
     * @author Byeong Yeol Jeon
     
     */
    ~C2SModuleSecureVariable() {
        C2SModuleSecureVariableRelease(&varId);
	}
	////////////////////////////////////////////////////////////
	/// equal
    /**
     * @brief equality operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	bool operator==(const U& _Right) const {
		return _Right == static_cast<U>(*this);
	}
    /**
     * @brief equality operator
     * @author Byeong Yeol Jeon
     
     */
	inline bool operator==(const C2SModuleSecureVariable& _Right) const {
		return static_cast<T>(*this) == static_cast<T>(_Right);
	}
    /**
     * @brief equality operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend bool operator==(const U& _Left, const C2SModuleSecureVariable& _Right) {
		return _Left == static_cast<U>(_Right);
	}
    
	////////////////////////////////////////////////////////////
	// not equal
    /**
     * @brief inequality operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	bool operator!=(const U& _Right) const {
		return _Right != static_cast<U>(*this);
	}
    /**
     * @brief inequality operator
     
     * @author Byeong Yeol Jeon
     
     */
	inline bool operator!=(const C2SModuleSecureVariable& _Right) const {
		return static_cast<T>(*this) != static_cast<T>(_Right);
	}
    /**
     * @brief inequality operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend bool operator!=(const U& _Left, const C2SModuleSecureVariable& _Right) {
		return _Left != static_cast<U>(_Right);
	}
    
	////////////////////////////////////////////////////////////
	// greater
    /**
     * @brief greater operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	bool operator>(const U& _Right) const {
		return static_cast<U>(*this) > _Right;
	}
    /**
     * @brief greater operator
     * @author Byeong Yeol Jeon
     
     */
	inline bool operator>(const C2SModuleSecureVariable& _Right) const {
		return static_cast<T>(*this) > static_cast<T>(_Right);
	}
    /**
     * @brief greater operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend bool operator>(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left > static_cast<U>(_Right);
	}
    
	////////////////////////////////////////////////////////////
	// greater equal
    /**
     * @brief greater equal operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	bool operator>=(const U& _Right) const {
		return static_cast<U>(*this) >= _Right;
	}
    /**
     * @brief greater equal operator
     
     * @author Byeong Yeol Jeon
     
     */
	inline bool operator>=(const C2SModuleSecureVariable& _Right) const {
		return static_cast<T>(*this) >= static_cast<T>(_Right);
	}
    /**
     * @brief greater equal operator
     
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend bool operator>=(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left >= static_cast<U>(_Right);
	}
    
	////////////////////////////////////////////////////////////
	// less
    /**
     * @brief less operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	bool operator<(const U& _Right) const {
		return static_cast<U>(*this) < _Right;
	}
    /**
     * @brief less operator
     * @author Byeong Yeol Jeon
     
     */
	inline bool operator<(const C2SModuleSecureVariable& _Right) const {
		return static_cast<T>(*this) < static_cast<T>(_Right);
	}
    /**
     * @brief less operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend bool operator<(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left < static_cast<U>(_Right);
	}
    
	////////////////////////////////////////////////////////////
	// less equal
    /**
     * @brief less equal operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	bool operator<=(const U& _Right) const {
		return static_cast<U>(*this) <= _Right;
	}
    /**
     * @brief less equal operator
     * @author Byeong Yeol Jeon
     
     */
	inline bool operator<=(const C2SModuleSecureVariable& _Right) const {
		return static_cast<T>(*this) <= static_cast<T>(_Right);
	}
    /**
     * @brief less equal operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend bool operator<=(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left <= static_cast<U>(_Right);
	}
    
	////////////////////////////////////////////////////////////
	// increment
    /**
     * @brief increment operator
     * @author Byeong Yeol Jeon
     
     */
	inline C2SModuleSecureVariable& operator++() {
		*this = static_cast<T>(*this) + 1;
		return *this;
	}
    /**
     * @brief increment operator
     * @author Byeong Yeol Jeon
     
     */
	inline C2SModuleSecureVariable operator++(int) {
		C2SModuleSecureVariable kTmp(*this);
		++(*this);
		return kTmp;
	}
    
	////////////////////////////////////////////////////////////
	// decrement
    /**
     * @brief decrement operator
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable& operator--() {
		*this = static_cast<T>(*this) - 1;
		return *this;
	}
    /**
     * @brief decrement operator
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable operator--(int) {
		C2SModuleSecureVariable kTmp(*this);
		--(*this);
		return kTmp;
	}
    
	////////////////////////////////////////////////////////////
	// assign
    /**
     * @brief assign operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable& operator=(const U& _Right) {
        T tmp(static_cast<T>(_Right));
        C2SModuleSecureVariableSet(varId, &tmp, sizeof(tmp));
		return *this;
	}
    /**
     * @brief assign operator
     * @author Byeong Yeol Jeon
     
     */
	inline C2SModuleSecureVariable& operator=(const C2SModuleSecureVariable& _Right) {
        C2SModuleSecureVariableCopy(varId, _Right.varId);
		return *this;
	}
    
	////////////////////////////////////////////////////////////
	// invert
    /**
     * @brief invert operator
     * @author Byeong Yeol Jeon
     
     */
	inline C2SModuleSecureVariable operator-() const {
		return C2SModuleSecureVariable(-static_cast<T>(*this));
	}
    
	////////////////////////////////////////////////////////////
	// add
    /**
     * @brief add operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable& operator+=(const U& _Right) {
		*this = static_cast<T>(*this) + static_cast<T>(_Right);
		return *this;
	}
    /**
     * @brief add operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	U operator+(const U& _Right) const {
		return static_cast<U>(*this) + _Right;
	}
    /**
     * @brief add operator
     
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable operator+(const C2SModuleSecureVariable& _Right) const {
		return C2SModuleSecureVariable(static_cast<T>(*this) + static_cast<T>(_Right));
	}
    /**
     * @brief add operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend U operator+(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left + static_cast<U>(_Right);
	}
	////////////////////////////////////////////////////////////
    
	////////////////////////////////////////////////////////////
	// subtract
    /**
     * @brief subtract operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable& operator-=(const U& _Right) {
		*this = static_cast<T>(*this) - static_cast<T>(_Right);
		return *this;
	}
    /**
     * @brief subtract operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	U operator-(const U& _Right) const {
		return static_cast<U>(*this) - _Right;
	}
    /**
     * @brief subtract operator
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable operator-(const C2SModuleSecureVariable& _Right) const {
		return C2SModuleSecureVariable(static_cast<T>(*this) - static_cast<T>(_Right));
	}
    /**
     * @brief subtract operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend U operator-(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left - static_cast<U>(_Right);
	}
	////////////////////////////////////////////////////////////
    
	////////////////////////////////////////////////////////////
	// multiply
    /**
     * @brief multiply operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable& operator*=(const U& _Right) {
		*this = static_cast<T>(*this) * static_cast<T>(_Right);
		return *this;
	}
    /**
     * @brief multiply operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	U operator*(const U& _Right) const {
		return static_cast<U>(*this) * _Right;
	}
    /**
     * @brief multiply operator
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable operator*(const C2SModuleSecureVariable& _Right) const {
		return C2SModuleSecureVariable(static_cast<T>(*this) * static_cast<T>(_Right));
	}
    /**
     * @brief multiply operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend U operator*(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left * static_cast<U>(_Right);
	}
	////////////////////////////////////////////////////////////
    
	////////////////////////////////////////////////////////////
	// divide
    /**
     * @brief divide operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable& operator/=(const U& _Right) {
		*this = static_cast<T>(*this) / static_cast<T>(_Right);
		return *this;
	}
    /**
     * @brief divide operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	U operator/(const U& _Right) const {
		return static_cast<U>(*this) / _Right;
	}
    /**
     
     * @brief divide operator
     * @author Byeong Yeol Jeon
     
     */
	C2SModuleSecureVariable operator/(const C2SModuleSecureVariable& _Right) const {
		return C2SModuleSecureVariable(static_cast<T>(*this) / static_cast<T>(_Right));
	}
    /**
     * @brief divide operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend U operator/(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left / static_cast<U>(_Right);
	}
	////////////////////////////////////////////////////////////
    
	////////////////////////////////////////////////////////////
	// remain
    /**
     * @brief remain operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	C2SModuleSecureVariable& operator%=(const U& _Right) {
		*this = static_cast<T>(*this) % static_cast<T>(_Right);
		return *this;
	}
    /**
     * @brief remain operator
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	U operator%(const U& _Right) const {
		return static_cast<U>(*this) % _Right;
	}
    /**
     * @brief remain operator
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	friend U operator%(U _Left, const C2SModuleSecureVariable& _Right) {
		return _Left % static_cast<U>(_Right);
	}
	////////////////////////////////////////////////////////////
    /**
     * @brief 특정 타입으로 변환
     
     * @author Byeong Yeol Jeon
     
     */
	template<typename U>
	operator U() const {
        T tmp;
        if(C2SModuleErrorCode_Broken == C2SModuleSecureVariableGet(varId, &tmp, sizeof(tmp)) )
        {
        	broken();
        }
        return static_cast<U>(tmp);
	}
private:
    /** 현재 암호화 오브젝트의 핸들 */
    C2SModuleSecureId varId;
    /** 무결성 체크 정책 */
    BrokenPolicy broken;
};
/**
 * @brief 8bit singed type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<int8_t> int8_e;
/**
 * @brief 16bit singed type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<int16_t> int16_e;
/**
 * @brief 32bit singed type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<int32_t> int32_e;
/**
 * @brief 64bit singed type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<int64_t> int64_e;
/**
 * @brief 8bit unsinged type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<uint8_t> uint8_e;
/**
 * @brief 16bit unsinged type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<uint16_t> uint16_e;
/**
 * @brief 32bit unsinged type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<uint32_t> uint32_e;
/**
 * @brief 64bit unsinged type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<uint64_t> uint64_e;
/**
 * @brief 32bit float type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<float> float_e;
/**
 * @brief 64bit float type
 * @author Byeong Yeol Jeon
 
 */
typedef C2SModuleSecureVariable<double> double_e;

#endif
#endif
